@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: client_notify.c,v 1.56 2002/05/23 12:38:56 priikone Exp $ */
+/* $Id: client_notify.c,v 1.57 2002/06/08 06:57:37 priikone Exp $ */
 /* This file includes the Notify packet handling. Notify packets are
    important packets sent by the server. They tell different things to the
    client such as nick changes, mode changes etc. */
@@ -1147,7 +1147,7 @@ void silc_client_notify_by_server(SilcClient client,
 
 	/* If same nick, the client was new to us and has become "present"
 	   to network.  Send NULL as nick to application. */
-	if (!strcmp(tmp, tmp_nick))
+	if (tmp_nick && !strcmp(tmp, tmp_nick))
 	  tmp = NULL;
 
 	silc_free(tmp_nick);
