@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: idlist.c,v 1.52 2002/05/18 08:32:21 priikone Exp $ */
+/* $Id: idlist.c,v 1.53 2002/05/30 11:20:29 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcclient.h"
@@ -1080,6 +1080,9 @@ void silc_client_nickname_format(SilcClient client,
       if (!client_entry->hostname)
 	break;
       len = strcspn(client_entry->hostname, ".");
+      i = strcspn(client_entry->hostname, "-");
+      if (i < len)
+        len = i;
       newnick = silc_realloc(newnick, sizeof(*newnick) * (off + len));
       memcpy(&newnick[off], client_entry->hostname, len);
       off += len;
